# encoding: utf-8

'''version
'''

__author__ = 'Chris Miles'
__copyright__ = '(c) Chris Miles 2009. All rights reserved.'
__id__ = '$Id: version.py 22 2009-08-21 03:23:25Z miles.chris $'
__url__ = '$URL: https://restez.googlecode.com/svn/trunk/restez/version.py $'

version = '0.3.0rc3'
